@@ -303,7 +303,7 @@ struct CompareWithCompareFunctionArguments {
     CompareWithCompareFunctionArguments(ExecState *e, ObjectImp *cf)
         : exec(e)
         , compareFunction(cf)
-        , globalObject(e->interpreter()->globalObject())
+        , globalObject(e->dynamicInterpreter()->globalObject())
     {
         arguments.append(Undefined());
         arguments.append(Undefined());
@@ -425,7 +425,7 @@ Value ArrayPrototypeImp::get(ExecState *exec, const Identifier &propertyName) co
 
 ArrayProtoFuncImp::ArrayProtoFuncImp(ExecState *exec, int i, int len)
   : InternalFunctionImp(
-    static_cast<FunctionPrototypeImp*>(exec->interpreter()->builtinFunctionPrototype().imp())
+    static_cast<FunctionPrototypeImp*>(exec->lexicalInterpreter()->builtinFunctionPrototype().imp())
     ), id(i)
 {
   Value protect(this);
@@ -476,7 +476,7 @@ Value ArrayProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &args
     break;
   }
   case Concat: {
-    Object arr = Object::dynamicCast(exec->interpreter()->builtinArray().construct(exec,List::empty()));
+    Object arr = Object::dynamicCast(exec->lexicalInterpreter()->builtinArray().construct(exec,List::empty()));
     int n = 0;
     Value curArg = thisObj;
     Object curObj = Object::dynamicCast(thisObj);
@@ -578,26 +578,35 @@ Value ArrayProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &args
     // http://developer.netscape.com/docs/manuals/js/client/jsref/array.htm#1193713 or 15.4.4.10
 
     // We return a new array
-    Object resObj = Object::dynamicCast(exec->interpreter()->builtinArray().construct(exec,List::empty()));
+    Object resObj = Object::dynamicCast(exec->lexicalInterpreter()->builtinArray().construct(exec,List::empty()));
     result = resObj;
-    int begin = args[0].toInteger(exec);
-    if ( begin < 0 )
-      begin = maxInt( begin + length, 0 );
-    else
-      begin = minInt( begin, length );
-    int end = length;
-    if (args[1].type() != UndefinedType)
-    {
+    double begin = args[0].toInteger(exec);
+    if (begin < 0) {
+      begin += length;
+      if (begin < 0)
+        begin = 0;
+    } else {
+      if (begin > length)
+        begin = length;
+    }
+    double end = length;
+    if (args[1].type() != UndefinedType) {
       end = args[1].toInteger(exec);
-      if ( end < 0 )
-        end = maxInt( end + length, 0 );
-      else
-        end = minInt( end, length );
+      if (end < 0) {
+        end += length;
+        if (end < 0)
+          end = 0;
+      } else {
+        if (end > length)
+          end = length;
+      }
     }
 
     //printf( "Slicing from %d to %d \n", begin, end );
     int n = 0;
-    for(int k = begin; k < end; k++, n++) {
+    int b = static_cast<int>(begin);
+    int e = static_cast<int>(end);
+    for(int k = b; k < e; k++, n++) {
       if (thisObj.hasProperty(exec, k)) {
         Value obj = thisObj.get(exec, k);
         resObj.put(exec, n, obj);
@@ -655,7 +664,7 @@ Value ArrayProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &args
                 List l;
                 l.append(jObj);
                 l.append(minObj);
-                cmp = sortFunction.call(exec, exec->interpreter()->globalObject(), l).toNumber(exec);
+                cmp = sortFunction.call(exec, exec->dynamicInterpreter()->globalObject(), l).toNumber(exec);
             } else {
               cmp = (jObj.toString(exec) < minObj.toString(exec)) ? -1 : 1;
             }
@@ -683,7 +692,7 @@ Value ArrayProtoFuncImp::call(ExecState *exec, Object &thisObj, const List &args
   }
   case Splice: {
     // 15.4.4.12 - oh boy this is huge
-    Object resObj = Object::dynamicCast(exec->interpreter()->builtinArray().construct(exec,List::empty()));
+    Object resObj = Object::dynamicCast(exec->lexicalInterpreter()->builtinArray().construct(exec,List::empty()));
     result = resObj;
     int begin = args[0].toUInt32(exec);
     if ( begin < 0 )
@@ -793,11 +802,11 @@ Object ArrayObjectImp::construct(ExecState *exec, const List &args)
       exec->setException(error);
       return error;
     }
-    return Object(new ArrayInstanceImp(exec->interpreter()->builtinArrayPrototype().imp(), n));
+    return Object(new ArrayInstanceImp(exec->lexicalInterpreter()->builtinArrayPrototype().imp(), n));
   }
 
   // otherwise the array is constructed with the arguments in it
-  return Object(new ArrayInstanceImp(exec->interpreter()->builtinArrayPrototype().imp(), args));
+  return Object(new ArrayInstanceImp(exec->lexicalInterpreter()->builtinArrayPrototype().imp(), args));
 }
 
 bool ArrayObjectImp::implementsCall() const
