@@ -20,8 +20,8 @@
  * 
  */
 /**
-    File Name:          15.5.4.11-2.js
-    ECMA Section:       15.5.4.11 String.prototype.toLowerCase()
+    File Name:          15.5.4.12-2.js
+    ECMA Section:       15.5.4.12 String.prototype.toUpperCase()
     Description:
 
     Returns a string equal in length to the length of the result of converting
@@ -32,7 +32,7 @@
     case the uppercase equivalent is used instead. (The canonical Unicode 2.0 case
     mapping shall be used, which does not depend on implementation or locale.)
 
-    Note that the toLowerCase function is intentionally generic; it does not require
+    Note that the toUpperCase function is intentionally generic; it does not require
     that its this value be a String object. Therefore it can be transferred to other
     kinds of objects for use as a method.
 
@@ -40,10 +40,10 @@
     Date:               12 november 1997
 */
 
-    var SECTION = "15.5.4.11-2";
+    var SECTION = "15.5.4.12-2";
     var VERSION = "ECMA_1";
     startTest();
-    var TITLE   = "String.prototype.toLowerCase()";
+    var TITLE   = "String.prototype.toUpperCase()";
 
     writeHeaderToLog( SECTION + " "+ TITLE);
 
@@ -54,23 +54,25 @@ function getTestCases() {
     var array = new Array();
     var item = 0;
 
-    // Georgian
-    // Range: U+10A0 to U+10FF
-    for ( var i = 0x10A0; i <= 0x10FF; i++ ) {
-        var U = new Unicode( i );
+    var TEST_STRING = "";
+    var EXPECT_STRING = "";
 
-/*
-        array[item++] = new TestCase(   SECTION,
-                                        "var s = new String( String.fromCharCode("+i+") ); s.toLowerCase()",
-                                        String.fromCharCode(U.lower),
-                                        eval("var s = new String( String.fromCharCode("+i+") ); s.toLowerCase()") );
-*/
-        array[item++] = new TestCase(   SECTION,
-                                        "var s = new String( String.fromCharCode("+i+") ); s.toLowerCase().charCodeAt(0)",
-                                        U.lower,
-                                        eval("var s = new String( String.fromCharCode(i) ); s.toLowerCase().charCodeAt(0)") );
+    // basic latin test
+
+    for ( var i = 0; i < 0x007A; i++ ) {
+        var u = new Unicode(i);
+        TEST_STRING += String.fromCharCode(i);
+        EXPECT_STRING += String.fromCharCode( u.upper );
     }
 
+    // don't print out the value of the strings since they contain control
+    // characters that break the driver
+    var isEqual = EXPECT_STRING == (new String( TEST_STRING )).toUpperCase();
+
+    array[item++] = new TestCase( SECTION,
+                                      "isEqual",
+                                      true,
+                                      isEqual);
     return array;
 }
 function test() {
@@ -513,4 +515,5 @@ function DecimalToHexString( n ) {
     }
 
     return h;
-}
\ No newline at end of file
+}
+
